@@ -5,6 +5,7 @@
 
 require_relative "bundler"
 require_relative "rubygems"
+require "pathname"
 
 module LogStash
   module Environment
@@ -16,7 +17,9 @@ module LogStash
     BUNDLE_DIR = ::File.join(LOGSTASH_HOME, "vendor", "bundle")
     GEMFILE_PATH = ::File.join(LOGSTASH_HOME, "Gemfile")
     LOCAL_GEM_PATH = ::File.join(LOGSTASH_HOME, 'vendor', 'local_gems')
-    CACHE_PATH = File.join(LOGSTASH_HOME, "vendor", "cache")
+    CACHE_PATH = ::File.join(LOGSTASH_HOME, "vendor", "cache")
+    LOCKFILE = Pathname.new(::File.join(LOGSTASH_HOME, "Gemfile.lock"))
+    GEMFILE = Pathname.new(::File.join(LOGSTASH_HOME, "Gemfile"))
 
     # @return [String] the ruby version string bundler uses to craft its gem path
     def gem_ruby_version
@@ -33,12 +36,14 @@ module LogStash
       RUBY_ENGINE
     end
 
-    def windows?
-      ::Gem.win_platform?
+    def oss_only?
+      return true if ENV['OSS']=="true"
+
+      !File.exists?(File.join(LogStash::Environment::LOGSTASH_HOME, "x-pack"))
     end
 
-    def jruby?
-      @jruby ||= !!(RUBY_PLATFORM == "java")
+    def windows?
+      ::Gem.win_platform?
     end
 
     def logstash_gem_home
@@ -56,7 +61,6 @@ module LogStash
   end
 end
 
-
 # when launched as a script, not require'd, (currently from bin/logstash and bin/logstash-plugin) the first
 # argument is the path of a Ruby file to require and a LogStash::Runner class is expected to be
 # defined and exposing the LogStash::Runner#main instance method which will be called with the current ARGV
@@ -66,5 +70,6 @@ if $0 == __FILE__
   require_relative "patches/jar_dependencies"
 
   require ARGV.shift
-  LogStash::Runner.new.main(ARGV)
+  exit_status = LogStash::Runner.run("bin/logstash", ARGV)
+  exit(exit_status || 0)
 end
