@@ -6,7 +6,7 @@ class LogStash::PluginManager::Command < Clamp::Command
 
   # If set in debug mode we will raise an exception and display the stacktrace
   def report_exception(readable_message, exception)
-    if ENV["DEBUG"]
+    if debug?
       raise exception
     else
       signal_error("#{readable_message}, message: #{exception.message}")
@@ -14,14 +14,13 @@ class LogStash::PluginManager::Command < Clamp::Command
   end
 
   def display_bundler_output(output)
-    if ENV['DEBUG'] && output
+    if debug? && output
       # Display what bundler did in the last run
       $stderr.puts("Bundler output")
       $stderr.puts(output)
     end
   end
 
-
   # Each plugin install for a gemfile create a path with a unique id.
   # we must clear what is not currently used in the 
   def remove_unused_locally_installed_gems!
@@ -36,4 +35,8 @@ class LogStash::PluginManager::Command < Clamp::Command
     require "pathname"
     ::Pathname.new(path).relative_path_from(::Pathname.new(LogStash::Environment::LOGSTASH_HOME)).to_s
   end
+
+  def debug?
+    ENV["DEBUG"]
+  end
 end
