@@ -8,8 +8,8 @@ class LogStash::PluginManager::List < LogStash::PluginManager::Command
 
   option "--installed", :flag, "List only explicitly installed plugins using bin/logstash-plugin install ...", :default => false
   option "--verbose", :flag, "Also show plugin version number", :default => false
-  option "--group", "NAME", "Filter plugins per group: input, output, filter or codec" do |arg|
-    raise(ArgumentError, "should be one of: input, output, filter or codec") unless ['input', 'output', 'filter', 'codec', 'pack'].include?(arg)
+  option "--group", "NAME", "Filter plugins per group: input, output, filter, codec or integration" do |arg|
+    raise(ArgumentError, "should be one of: input, output, filter, codec, integration") unless ['input', 'output', 'filter', 'codec', 'pack', 'integration'].include?(arg)
     arg
   end
 
@@ -22,6 +22,16 @@ class LogStash::PluginManager::List < LogStash::PluginManager::Command
       line = "#{spec.name}"
       line += " (#{spec.version})" if verbose?
       puts(line)
+      if spec.metadata.fetch("logstash_group", "") == "integration"
+        integration_plugins = spec.metadata.fetch("integration_plugins", "").split(",")
+        integration_plugins.each_with_index do |integration_plugin, i|
+          if i == integration_plugins.size - 1
+            puts(" └── #{integration_plugin}")
+          else
+            puts(" ├── #{integration_plugin}")
+          end
+        end
+      end
     end
   end
 
@@ -32,10 +42,19 @@ class LogStash::PluginManager::List < LogStash::PluginManager::Command
 
                           # apply filters
                           specs = specs.select{|spec| gemfile.find(spec.name)} if installed?
-                          specs = specs.select{|spec| spec.name =~ /#{plugin}/i} if plugin
+                          specs = specs.select{|spec| spec_matches_search?(spec) } if plugin
                           specs = specs.select{|spec| spec.metadata['logstash_group'] == group} if group
 
                           specs
                         end
   end
+
+  def spec_matches_search?(spec)
+    return true if spec.name =~ /#{plugin}/i
+    if LogStash::PluginManager.integration_plugin_spec?(spec)
+      LogStash::PluginManager.integration_plugin_provides(spec).any? do |provided_plugin|
+        provided_plugin =~ /#{plugin}/i
+      end
+    end
+  end
 end # class Logstash::PluginManager
