@@ -1,5 +1,6 @@
 # encoding: utf-8
 require "rubygems/package"
+require_relative "../bootstrap/patches/remote_fetcher"
 
 module LogStash::PluginManager
 
@@ -50,9 +51,8 @@ module LogStash::PluginManager
   # @option options [Boolean] :pre Include pre release versions in the search (default: false)
   # @return [Hash] The plugin version information as returned by rubygems
   def self.fetch_latest_version_info(plugin, options={})
-    require "gems"
     exclude_prereleases =  options.fetch(:pre, false)
-    versions = Gems.versions(plugin)
+    versions = LogStash::Rubygems.versions(plugin)
     raise ValidationError.new("Something went wrong with the validation. You can skip the validation with the --no-verify option") if !versions.is_a?(Array) || versions.empty?
     versions = versions.select { |version| !version["prerelease"] } if !exclude_prereleases
     versions.first
@@ -78,7 +78,7 @@ module LogStash::PluginManager
   end
 
   # retrieve gem specs for all or specified name valid logstash plugins locally installed
-  # @param name [String] specific plugin name to find or nil for all plungins
+  # @param name [String] specific plugin name to find or nil for all plugins
   # @return [Array<Gem::Specification>] all local logstash plugin gem specs
   def self.find_plugins_gem_specs(name = nil)
     specs = name ? Gem::Specification.find_all_by_name(name) : Gem::Specification.find_all
@@ -86,7 +86,7 @@ module LogStash::PluginManager
   end
 
   # list of all locally installed plugins specs specified in the Gemfile.
-  # note that an installed plugin dependecies like codecs will not be listed, only those
+  # note that an installed plugin dependencies like codecs will not be listed, only those
   # specifically listed in the Gemfile.
   # @param gemfile [LogStash::Gemfile] the gemfile to validate against
   # @return [Array<Gem::Specification>] list of plugin specs
@@ -98,13 +98,37 @@ module LogStash::PluginManager
 
   # @param plugin [String] plugin name
   # @param gemfile [LogStash::Gemfile] the gemfile to validate against
-  # @return [Boolean] true if the plugin is an installed logstash plugin and spefificed in the Gemfile
+  # @return [Boolean] true if the plugin is an installed logstash plugin and specified in the Gemfile
   def self.installed_plugin?(plugin, gemfile)
     !!gemfile.find(plugin) && find_plugins_gem_specs(plugin).any?
   end
 
+  # @param spec [Gem::Specification] plugin specification
+  # @return [Boolean] true if the gemspec is from an integration plugin
+  def self.integration_plugin_spec?(spec)
+    spec.metadata &&
+      spec.metadata["logstash_plugin"] == "true" &&
+      spec.metadata["logstash_group"] == "integration"
+  end
+
+  # @param spec [Gem::Specification] plugin specification
+  # @return [Array] array of [plugin name] representing plugins a given integration plugin provides
+  def self.integration_plugin_provides(spec)
+    spec.metadata["integration_plugins"].split(",")
+  end
+
+  # @param name [String] plugin name
+  # @return [Gem::Specification] Gem specification of integration plugin that provides plugin
+  def self.which_integration_plugin_provides(name, gemfile)
+    all_installed_plugins_gem_specs(gemfile) \
+      .select {|spec| integration_plugin_spec?(spec) }
+      .find do |integration_plugin|
+        integration_plugin_provides(integration_plugin).any? {|plugin| plugin == name }
+      end
+  end
+
   # @param plugin_list [Array] array of [plugin name, version] tuples
-  # @return [Array] array of [plugin name, version, ...] tuples when duplciate names have been merged and non duplicate version requirements added
+  # @return [Array] array of [plugin name, version, ...] tuples when duplicate names have been merged and non duplicate version requirements added
   def self.merge_duplicates(plugin_list)
 
     # quick & dirty naive dedup for now
